package java.awt;

import java.awt.image.DirectColorModel;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.DataBufferInt;
import java.lang.ref.SoftReference;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.Raster;
import java.lang.ref.WeakReference;
import java.awt.image.ColorModel;

abstract class MultipleGradientPaintContext implements PaintContext
{
    protected ColorModel model;
    private static ColorModel xrgbmodel;
    protected static ColorModel cachedModel;
    protected static WeakReference<Raster> cached;
    protected Raster saved;
    protected MultipleGradientPaint.CycleMethod cycleMethod;
    protected MultipleGradientPaint.ColorSpaceType colorSpace;
    protected float a00;
    protected float a01;
    protected float a10;
    protected float a11;
    protected float a02;
    protected float a12;
    protected boolean isSimpleLookup;
    protected int fastGradientArraySize;
    protected int[] gradient;
    private int[][] gradients;
    private float[] normalizedIntervals;
    private float[] fractions;
    private int transparencyTest;
    private static final int[] SRGBtoLinearRGB;
    private static final int[] LinearRGBtoSRGB;
    protected static final int GRADIENT_SIZE = 256;
    protected static final int GRADIENT_SIZE_INDEX = 255;
    private static final int MAX_GRADIENT_ARRAY_SIZE = 5000;
    
    protected MultipleGradientPaintContext(MultipleGradientPaint multipleGradientPaint, ColorModel colorModel, Rectangle rectangle, Rectangle2D rectangle2D, AffineTransform affineTransform, RenderingHints renderingHints, float[] fractions, Color[] array, MultipleGradientPaint.CycleMethod cycleMethod, MultipleGradientPaint.ColorSpaceType colorSpace) {
        if (rectangle == null) {
            throw new NullPointerException("Device bounds cannot be null");
        }
        if (rectangle2D == null) {
            throw new NullPointerException("User bounds cannot be null");
        }
        if (affineTransform == null) {
            throw new NullPointerException("Transform cannot be null");
        }
        if (renderingHints == null) {
            throw new NullPointerException("RenderingHints cannot be null");
        }
        try {
            affineTransform.invert();
        }
        catch (NoninvertibleTransformException ex) {
            affineTransform = new AffineTransform();
        }
        double[] flatmatrix = new double[6];
        affineTransform.getMatrix(flatmatrix);
        this.a00 = (float)flatmatrix[0];
        this.a10 = (float)flatmatrix[1];
        this.a01 = (float)flatmatrix[2];
        this.a11 = (float)flatmatrix[3];
        this.a02 = (float)flatmatrix[4];
        this.a12 = (float)flatmatrix[5];
        this.cycleMethod = cycleMethod;
        this.colorSpace = colorSpace;
        this.fractions = fractions;
        int[] gradient = (multipleGradientPaint.gradient != null) ? ((int[])multipleGradientPaint.gradient.get()) : null;
        int[][] gradients = (multipleGradientPaint.gradients != null) ? ((int[][])multipleGradientPaint.gradients.get()) : ((int[][])null);
        if (gradient == null && gradients == null) {
            this.calculateLookupData(array);
            multipleGradientPaint.model = this.model;
            multipleGradientPaint.normalizedIntervals = this.normalizedIntervals;
            multipleGradientPaint.isSimpleLookup = this.isSimpleLookup;
            if (this.isSimpleLookup) {
                multipleGradientPaint.fastGradientArraySize = this.fastGradientArraySize;
                multipleGradientPaint.gradient = new SoftReference(this.gradient);
            }
            else {
                multipleGradientPaint.gradients = new SoftReference(this.gradients);
            }
        }
        else {
            this.model = multipleGradientPaint.model;
            this.normalizedIntervals = multipleGradientPaint.normalizedIntervals;
            this.isSimpleLookup = multipleGradientPaint.isSimpleLookup;
            this.gradient = gradient;
            this.fastGradientArraySize = multipleGradientPaint.fastGradientArraySize;
            this.gradients = gradients;
        }
        return;
    }
    
    private void calculateLookupData(Color[] array) {
        Color[] array2;
        if (this.colorSpace == MultipleGradientPaint.ColorSpaceType.LINEAR_RGB) {
            array2 = new Color[array.length];
            for (int i = 0; i < array.length; ++i) {
                int rgb = array[i].getRGB();
                array2[i] = new Color(MultipleGradientPaintContext.SRGBtoLinearRGB[rgb >> 16 & 0xFF], MultipleGradientPaintContext.SRGBtoLinearRGB[rgb >> 8 & 0xFF], MultipleGradientPaintContext.SRGBtoLinearRGB[rgb & 0xFF], rgb >>> 24);
            }
        }
        else {
            array2 = array;
        }
        this.normalizedIntervals = new float[this.fractions.length - 1];
        for (int j = 0; j < this.normalizedIntervals.length; ++j) {
            this.normalizedIntervals[j] = this.fractions[j + 1] - this.fractions[j];
        }
        this.transparencyTest = -16777216;
        this.gradients = new int[this.normalizedIntervals.length][];
        float n = 1.0f;
        for (int k = 0; k < this.normalizedIntervals.length; ++k) {
            n = ((n > this.normalizedIntervals[k]) ? this.normalizedIntervals[k] : n);
        }
        int n2 = 0;
        for (int l = 0; l < this.normalizedIntervals.length; ++l) {
            n2 = (int)((float)n2 + this.normalizedIntervals[l] / n * 256.0f);
        }
        if (n2 > 5000) {
            this.calculateMultipleArrayGradient(array2);
        }
        else {
            this.calculateSingleArrayGradient(array2, n);
        }
        if (this.transparencyTest >>> 24 == 255) {
            this.model = MultipleGradientPaintContext.xrgbmodel;
        }
        else {
            this.model = ColorModel.getRGBdefault();
        }
    }
    
    private void calculateSingleArrayGradient(Color[] array, float n) {
        this.isSimpleLookup = true;
        int n2 = 1;
        int rgb;
        int rgb2;
        for (int i = 0; i < this.gradients.length; this.transparencyTest &= rgb, this.transparencyTest &= rgb2, ++i) {
            int n3 = (int)(this.normalizedIntervals[i] / n * 255.0f);
            n2 += n3;
            this.gradients[i] = new int[n3];
            rgb = array[i].getRGB();
            rgb2 = array[i + 1].getRGB();
            this.interpolate(rgb, rgb2, this.gradients[i]);
        }
        this.gradient = new int[n2];
        for (int n4 = 0, j = 0; j < this.gradients.length; n4 += this.gradients[j].length, ++j) {
            System.arraycopy(this.gradients[j], 0, this.gradient, n4, this.gradients[j].length);
        }
        this.gradient[this.gradient.length - 1] = array[array.length - 1].getRGB();
        if (this.colorSpace == MultipleGradientPaint.ColorSpaceType.LINEAR_RGB) {
            for (int k = 0; k < this.gradient.length; ++k) {
                this.gradient[k] = this.convertEntireColorLinearRGBtoSRGB(this.gradient[k]);
            }
        }
        this.fastGradientArraySize = this.gradient.length - 1;
    }
    
    private void calculateMultipleArrayGradient(Color[] array) {
        this.isSimpleLookup = false;
        int rgb;
        int rgb2;
        for (int i = 0; i < this.gradients.length; this.transparencyTest &= rgb, this.transparencyTest &= rgb2, ++i) {
            this.gradients[i] = new int[256];
            rgb = array[i].getRGB();
            rgb2 = array[i + 1].getRGB();
            this.interpolate(rgb, rgb2, this.gradients[i]);
        }
        if (this.colorSpace == MultipleGradientPaint.ColorSpaceType.LINEAR_RGB) {
            for (int j = 0; j < this.gradients.length; ++j) {
                for (int k = 0; k < this.gradients[j].length; ++k) {
                    this.gradients[j][k] = this.convertEntireColorLinearRGBtoSRGB(this.gradients[j][k]);
                }
            }
        }
    }
    
    private void interpolate(int n, int n2, int[] array) {
        float n3 = 1.0f / (float)array.length;
        int n4 = n >> 24 & 0xFF;
        int n5 = n >> 16 & 0xFF;
        int n6 = n >> 8 & 0xFF;
        int n7 = n & 0xFF;
        int n8 = (n2 >> 24 & 0xFF) - n4;
        int n9 = (n2 >> 16 & 0xFF) - n5;
        int n10 = (n2 >> 8 & 0xFF) - n6;
        int n11 = (n2 & 0xFF) - n7;
        for (int i = 0; i < array.length; ++i) {
            array[i] = ((int)((double)((float)n4 + (float)(i * n8) * n3) + 0.5) << 24 | (int)((double)((float)n5 + (float)(i * n9) * n3) + 0.5) << 16 | (int)((double)((float)n6 + (float)(i * n10) * n3) + 0.5) << 8 | (int)((double)((float)n7 + (float)(i * n11) * n3) + 0.5));
        }
    }
    
    private int convertEntireColorLinearRGBtoSRGB(int n) {
        return (n >> 24 & 0xFF) << 24 | MultipleGradientPaintContext.LinearRGBtoSRGB[n >> 16 & 0xFF] << 16 | MultipleGradientPaintContext.LinearRGBtoSRGB[n >> 8 & 0xFF] << 8 | MultipleGradientPaintContext.LinearRGBtoSRGB[n & 0xFF];
    }
    
    protected final int indexIntoGradientsArrays(float n) {
        if (this.cycleMethod == MultipleGradientPaint.CycleMethod.NO_CYCLE) {
            if (n > 1.0f) {
                n = 1.0f;
            }
            else if (n < 0.0f) {
                n = 0.0f;
            }
        }
        else if (this.cycleMethod == MultipleGradientPaint.CycleMethod.REPEAT) {
            n -= (float)(int)n;
            if (n < 0.0f) {
                ++n;
            }
        }
        else {
            if (n < 0.0f) {
                n = -n;
            }
            int n2 = (int)n;
            n -= (float)n2;
            if ((n2 & 0x1) == 0x1) {
                n = 1.0f - n;
            }
        }
        if (this.isSimpleLookup) {
            return this.gradient[(int)(n * (float)this.fastGradientArraySize)];
        }
        int i = 0;
        while (i < this.gradients.length) {
            if (n < this.fractions[i + 1]) {
                return this.gradients[i][(int)((n - this.fractions[i]) / this.normalizedIntervals[i] * 255.0f)];
            }
            ++i;
        }
        return this.gradients[this.gradients.length - 1][255];
    }
    
    private static int convertSRGBtoLinearRGB(int n) {
        float n2 = (float)n / 255.0f;
        float n3;
        if (n2 <= 0.04045f) {
            n3 = n2 / 12.92f;
        }
        else {
            n3 = (float)Math.pow(((double)n2 + 0.055) / 1.055, 2.4);
        }
        return Math.round(n3 * 255.0f);
    }
    
    private static int convertLinearRGBtoSRGB(int n) {
        float n2 = (float)n / 255.0f;
        float n3;
        if ((double)n2 <= 0.0031308) {
            n3 = n2 * 12.92f;
        }
        else {
            n3 = 1.055f * (float)Math.pow((double)n2, 0.4166666666666667) - 0.055f;
        }
        return Math.round(n3 * 255.0f);
    }
    
    @Override
    public final Raster getRaster(int n, int n2, int n3, int n4) {
        Raster saved = this.saved;
        if (saved == null || saved.getWidth() < n3 || saved.getHeight() < n4) {
            saved = getCachedRaster(this.model, n3, n4);
            this.saved = saved;
        }
        DataBufferInt dataBufferInt = (DataBufferInt)saved.getDataBuffer();
        this.fillRaster(dataBufferInt.getData(0), dataBufferInt.getOffset(), ((SinglePixelPackedSampleModel)saved.getSampleModel()).getScanlineStride() - n3, n, n2, n3, n4);
        return saved;
    }
    
    protected abstract void fillRaster(int[] p0, int p1, int p2, int p3, int p4, int p5, int p6);
    
    private static synchronized Raster getCachedRaster(ColorModel colorModel, int w, int h) {
        if (colorModel == MultipleGradientPaintContext.cachedModel && MultipleGradientPaintContext.cached != null) {
            Raster raster = (Raster)MultipleGradientPaintContext.cached.get();
            if (raster != null && raster.getWidth() >= w && raster.getHeight() >= h) {
                MultipleGradientPaintContext.cached = null;
                return raster;
            }
        }
        return colorModel.createCompatibleWritableRaster(w, h);
    }
    
    private static synchronized void putCachedRaster(ColorModel cachedModel, Raster referent) {
        if (MultipleGradientPaintContext.cached != null) {
            Raster raster = (Raster)MultipleGradientPaintContext.cached.get();
            if (raster != null) {
                int width = raster.getWidth();
                int height = raster.getHeight();
                int width2 = referent.getWidth();
                int height2 = referent.getHeight();
                if (width >= width2 && height >= height2) {
                    return;
                }
                if (width * height >= width2 * height2) {
                    return;
                }
            }
        }
        MultipleGradientPaintContext.cachedModel = cachedModel;
        MultipleGradientPaintContext.cached = new WeakReference(referent);
    }
    
    @Override
    public final void dispose() {
        if (this.saved != null) {
            putCachedRaster(this.model, this.saved);
            this.saved = null;
        }
    }
    
    @Override
    public final ColorModel getColorModel() {
        return this.model;
    }
    
    static {
        MultipleGradientPaintContext.xrgbmodel = new DirectColorModel(24, 16711680, 65280, 255);
        SRGBtoLinearRGB = new int[256];
        LinearRGBtoSRGB = new int[256];
        for (int i = 0; i < 256; ++i) {
            MultipleGradientPaintContext.SRGBtoLinearRGB[i] = convertSRGBtoLinearRGB(i);
            MultipleGradientPaintContext.LinearRGBtoSRGB[i] = convertLinearRGBtoSRGB(i);
        }
    }
}
